const Promise = require('bluebird');
const baseClass = require('../lib/baseClass');

module.exports = class Cache extends baseClass {
    /**
     * @description 设置缓存
     * @param {string} id
     * @param {any} data
     * @param {int} [expires]
     */
    setCache(id, data, expires) {
        if (!expires) expires = 60;
        return new Promise((resolve, reject) => {
            this.request().post(`${this.getPath()}/${id}/${expires}`, {
                data: data
            }).then(data => {
                resolve(data.data);
            }).catch(err => {
                reject(err);
            });
        });
    }

    /**
     * @description 获取缓存
     * @param {string} id
     */
    getCache(id) {
        return new Promise((resolve, reject) => {
            this.request().get(`${this.getPath()}/${id}`).then(data => {
                resolve(data.data);
            }).catch(err => {
                reject(err);
            });
        });
    }

    /**
     * @description 移除缓存
     * @param {string} id
     */
    removeCache(id) {
        return new Promise((resolve, reject) => {
            this.request().del(`${this.getPath()}/${id}`).then(() => {
                resolve();
            }).catch(err => {
                reject(err);
            });
        });
    }
};