const SetHeader = require('./lib/setHeader');
const Schemas = require('./classes/Schemas');
const Objects = require('./classes/Objects');
const Logs = require('./classes/Logs');
const Users = require('./classes/Users');
const Roles = require('./classes/Roles');
const Orders = require('./classes/Orders');
const Captcha = require('./classes/Captcha');
const Im = require('./classes/Im');
const Payment = require('./classes/Payment');
const Utility = require('./classes/Utility');
const Files = require('./classes/Files');
const Sms = require('./classes/Sms');
const Accounts = require('./classes/Accounts');
const Cache = require('./classes/Cache');
const Excel = require('./classes/Excel');

let sdk = function (appId, appKey) {
    this.headers = SetHeader('id', appId);
    this.headers = SetHeader('key', appKey, this.headers);
};

sdk.prototype = {
    /**
     * @return {Schemas}
     */
    schema: function () {
        return new Schemas('/schemas', this.headers);
    },
    /**
     * @return {Objects}
     */
    object: function (schemaName) {
        let objectsClass = new Objects('/objects', this.headers);
        objectsClass.setSchema(schemaName);
        return objectsClass;
    },
    log: function () {
        return new Logs('/logs', this.headers);
    },
    user: function () {
        return new Users('/users', this.headers);
    },
    role: function () {
        return new Roles('/roles', this.headers);
    },
    order: function () {
        return new Orders('/orders', this.headers);
    },
    captcha: function () {
        return new Captcha('/captcha', this.headers);
    },
    im: function () {
        return new Im('/im', this.headers);
    },
    payment: function () {
        return new Payment('/payment', this.headers);
    },
    utility: function () {
        return new Utility('/', this.headers);
    },
    file: function () {
        return new Files('/files', this.headers);
    },
    sms: function () {
        return new Sms('/sms', this.headers);
    },
    account: function () {
        return new Accounts('/account', this.headers);
    },
    cache: function () {
        return new Cache('/cache', this.headers);
    },
    excel: function () {
        return new Excel('/excel', this.headers);
    }
};

module.exports = sdk;